<?php

namespace Fastrack\Property;

use Fastrack\Gallery\Gallery;
use Fastrack\Utilities\ModelUtilities;
use Fastrack\Utilities\Slug;
use PlatoCreative\Imgix\Imgix;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Forms\FieldList;
use SilverStripe\ORM\ArrayList;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\HasManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Security\Security;

/**
 * @property string $Name
 * @property string $Title
 * @property string $Slug
 * @property string $Excerpt
 * @property string $Content
 * @property string $NumberOfSpaces
 * @property string $NumberOfGuests
 * @property string $Size
 * @property bool $CateringAvailable
 * @property string $SortOrder
 * @method Property Property()
 * @method Imgix Image()
 * @method HasManyList Galleries()
 */
class EventRoom extends DataObject implements PermissionProvider
{
	private static string $table_name = 'event_room';

	private static array $db = [
		'Title' => 'Varchar',
		'Name' => 'Varchar',
		'NumberOfSpaces' => 'Varchar',
		'NumberOfGuests' => 'Varchar',
		'Size' => 'Varchar',
		'CateringAvailable' => 'Boolean',
		'SortOrder' => 'Int',
	];

	private static array $has_one = [
		'Property' => Property::class,
	];

	private static array $has_many = [
		'Galleries' => Gallery::class,
	];

	private static array $indexes = [
		'Slug' => true,
		'SortOrder' => true,
	];

	private static array $translate = [
		'Title',
		'Name',
	];

	private static string $default_sort = 'SortOrder';

	public function getCMSFields()
	{
		$fields = parent::getCMSFields();
		ModelUtilities::hideHasOneFields($this, $fields);
		ModelUtilities::hideConfigFields($this, $fields);
		$fields->removeByName('Title');
		$fields->removeByName('SortOrder');
		return $fields;
	}

	public function onBeforeWrite() {
		$this->Title = $this->Name;
		$this->Slug = Slug::generateSlug($this->Name, $this->Slug);
		Gallery::createDefaultGalleries($this);
		parent::onBeforeWrite();
	}

	public function Link(): string
	{
		$link = new ArrayList([$this->Property()->EventRoomsLink(), $this->Slug]);
		$this->extend('updateLink', $link);
		return Controller::join_links($link->toArray());
	}

	public function CMSEditLink(): string
	{
		return Controller::join_links([
			$this->Property()->CMSEditLink(),
			'ItemEditForm',
			'field',
			'EventRooms',
			'item',
			$this->ID,
		]);
	}

	public function WorkflowLink(): string
	{
		return Controller::join_links([Director::absoluteBaseURL(), $this->CMSEditLink()]);
	}

	public function providePermissions(): array
	{
		$permissions = [
			'MANAGE_EVENT_ROOMS' => [
				'name' => 'Manage event rooms',
				'category' => 'Properties',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null)
	{
		if (!$member) {
			if (!Security::getCurrentUser()) {
				return $this->Status === 'Disabled' ? false : true;
			}
		}
		return Permission::check(['MANAGE_EVENT_ROOMS']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_EVENT_ROOMS']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_EVENT_ROOMS']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_EVENT_ROOMS']);
	}

	public function AbsoluteLink()
	{
		return Director::absoluteURL($this->Link());
	}

	public function useInheritedWorkflow()
	{
		return true;
	}

	public function workflowParent()
	{
		return $this->Property();
	}

}
