<?php

namespace Fastrack\Property;

use Fastrack\Gallery\Gallery;
use Fastrack\Utilities\ModelUtilities;
use PlatoCreative\Imgix\Imgix;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Forms\FieldList;
use SilverStripe\ORM\ArrayList;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\HasManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Security\Security;

/**
 * @property string Title
 * @property string Name
 * @property string Slug
 * @property string Excerpt
 * @property string Content
 * @property int SortOrder
 * @method Property Property()
 * @method Imgix Image()
 * @method HasManyList Gallery()
 */
class Restaurant extends DataObject implements PermissionProvider
{
	private static string $table_name = 'restaurant';

	private static array $db = [
		'Title' => 'Varchar(255)',
		'Name' => 'Varchar(255)',
		'SortOrder' => 'Int',
	];

	private static array $has_one = [
		'Property' => Property::class,
	];

	private static array $has_many = [
		'Galleries' => Gallery::class,
	];

	private static array $translate = [
		'Title',
		'Name',
	];

	private static array $indexes = [
		'SortOrder' => true,
	];

	private static string $default_sort = 'SortOrder';

	protected function onBeforeWrite()
	{
		$this->Title = $this->Name;
		Gallery::createDefaultGalleries($this);
		parent::onBeforeWrite();
	}

	public function getCMSFields(): FieldList
	{
		$fields = parent::getCMSFields();
		$fields->removeByName('Title');
		$fields->removeByName('SortOrder');
		ModelUtilities::hideConfigFields($this, $fields);
		return $fields;
	}

	public function Link(): string
	{
		$link = new ArrayList([$this->Property()->RestaurantLink(), $this->Slug]);
		$this->extend('updateLink', $link);
		return Controller::join_links($link->toArray());
	}

	public function CMSEditLink(): string
	{
		return Controller::join_links([
			$this->Property()->CMSEditLink(),
			'ItemEditForm',
			'field',
			'Restaurants',
			'item',
			$this->ID,
		]);
	}

	public function WorkflowLink(): string
	{
		return Controller::join_links([Director::absoluteBaseURL(), $this->CMSEditLink()]);
	}

	public function providePermissions(): array
	{
		$permissions = [
			'MANAGE_RESTAURANTS' => [
				'name' => 'Manage restaurants',
				'category' => 'Properties',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null)
	{
		if (!$member) {
			if (!Security::getCurrentUser()) {
				return true;
			}
		}
		return Permission::check(['MANAGE_RESTAURANTS']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_RESTAURANTS']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_RESTAURANTS']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_RESTAURANTS']);
	}

	public function useInheritedWorkflow(): bool
	{
		return true;
	}

	public function workflowParent(): Property
	{
		return $this->Property();
	}
}
