<?php

namespace Fastrack\Property;

use Fastrack\Utilities\Api;
use SilverStripe\CMS\Controllers\ContentController;
use SilverStripe\Control\HTTPResponse;

class RestaurantApiController extends ContentController
{

	public const RESTAURANT_LISTING = '/api/restaurants';

	/**
	 * @config
	 */
	private static string $gallery_key = 'gallery';

	/**
	 * @config
	 */
	private static int $max_gallery_images = 4;

	private static array $allowed_actions = [
		'restaurants',
	];

	private static array $url_handlers = [
		'' => 'restaurants',
	];

	public function restaurants(): HTTPResponse
	{
		$CACHE_KEY = 'restaurants';
		$restaurants = Api::fetchFromCache($CACHE_KEY, []);
		if (!count($restaurants)) {
			foreach (Restaurant::get() as $restaurant) {
				$restaurants[] = $this->buildRestaurant($restaurant);
			}
			Api::setCache($CACHE_KEY, $restaurants);
		}
		return Api::constructResponse($this, $restaurants);
	}

	private function buildRestaurant(Restaurant $restaurant): array
	{
		$data = [
			'id' => $restaurant->ID,
			'name' => $restaurant->Name,
			'slug' => $restaurant->Slug,
			'excerpt' => $restaurant->Excerpt,
			'content' => $restaurant->Content,
			'phone' => $restaurant->getField('Phone'),
			'email' => $restaurant->getField('Email'),
			'image' => Api::buildImage($restaurant->Image()),
			'slides' => Api::buildGallery($restaurant, $this->config()->get('gallery_key'), (int) $this->config()->get('max_gallery_images'))
		];
		$this->extend('updateBuildRestaurant', $data, $restaurant);
		return $data;
	}

}
